==> node_link_report-8.x-1.0.tar.gz: node_link_report/src/Plugin/Block/NodeLinkReport.php <==
<?php

namespace Drupal\node_link_report\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\node\NodeInterface;

/**
 * Provides a Node Report Block.
 *
 * @Block(
 *   id = "node_link_report_block",
 *   admin_label = @Translation("Node Link Report"),
 * )
 */
class NodeLinkReport extends BlockBase {

  /**
   * @var object $nlr_config
   *  Configuration settings object for node_link_report module.
   */
private $nlr_config;

/**
 * @var NodeInterface $node
 *   The node object for the current node.
 */
private $node = NULL;

  /**
   * NodeLinkReport constructor.
   */
  public function __construct() {
    $this->nlr_config = \Drupal::config('node_link_report.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $nid = $this->getNode()->id();
    $cid = "node_link_report_{$nid}";
    // Check for cached data.
    $cached_data  = \Drupal::cache()->get($cid);
    if (!empty($cached_data) && (!$this->isPreview())) {
      $renderable = $cached_data->data;
    }
    else {
      // Cache unavailable so generate the data.
      $links = $this->checkLinks();
      $renderable = [
        '#theme' => 'node_link_report_block',
        '#node_id' => $nid,
        '#bad_links' => $links['bad'],
        '#display_good_links' => $this->nlr_config->get('enable_reporting_good_links'),
        '#good_links' => $links['good'],
        '#unpublished_links' => $links['unpublished'],
      ];

      if (!$this->isPreview()) {
        // This is not a preview, so cache the data. for 24 hours.
        $expire_time = time() + 24 * 60 * 60;
        $tags = [
          'node_link_report',
          "node:{$nid}",
        ];
        \Drupal::cache()->set($cid, $renderable, $expire_time, $tags);
      }
    }

    $rendered = \Drupal::service('renderer')->render($renderable);
    return [
      '#markup' => $rendered,
    ];
  }

  /**
   * Checks the links of any found in the node content.
   *
   * @return array
   *   Array with the following keys 'bad', 'good', and 'unpublished'.
   */
  private function checkLinks() {
    $links = [
      'bad' => [],
      'good' => [],
      'unpublished' => [],
    ];
    $node = clone $this->getNode();
    $html = $this->getNodeHtml($node);
    //Instantiate the DOMDocument class.
    $htmlDom = new \DOMDocument;
    //Parse the HTML of the page using DOMDocument::loadHTML
    @$htmlDom->loadHTML($html);

    //Extract the links from the HTML.
    $urls = $htmlDom->getElementsByTagName('a');
    //Dedupe the urls, by url.
    $deduped_urls = [];
    foreach ($urls as $url) {
      //Get the link text.
      $linkText = $url->nodeValue;
      //Get the link in the href attribute.
      $linkHref = $url->getAttribute('href');
      if (!empty($linkHref) && $this->isAllowed($linkHref)) {
        if (isset($deduped_urls[$linkHref])) {
          $deduped_urls[$linkHref] .= "[{$linkText}] ";
        }
        else {
          $deduped_urls[$linkHref] = "[{$linkText}] ";
        }

      }
    }
    foreach ($deduped_urls as $url => $text) {
      // Curl the href to get the response.
      $exists = $this->curlUrl($url);
      if ($exists) {
        // The url is accessible.
        $links['good'][$url] = $text;
      }
      else {
        // The url is not accessible and is either bad or unpublished.
        // @TODO Need to check to see if it is unpublished drupal.
        if ($this->isExternal($url)) {
          // This is definitely external and bad.
          $links['bad'][$url] = $text;
        }
        else {
          // The internal link may be bad or unpublished.
          if ($this->isUnpublished($url)) {
            // It is an url to an unpublished page.
            $links['unpublished'][$url] = $text;
          }
          else {
            // It is not unpublished, just bad.
            $links['bad'][$url] = $text;
          }
        }
      }
    }

    return $links;
  }

  /**
   * Looks up to see if an internal link is unpublished.
   *
   * @param string $url
   *   A url to lookup.
   *
   * @return bool
   *   TRUE if the link points to an unpublished entity.  FALSE otherwise.
   */
  private function isUnpublished($url) {
    // Look at the path.
    $path = parse_url($url, PHP_URL_PATH);
    // Determine if it is a node/NID path.
    if (!(strpos($path, 'node') === 0) || !(strpos('taxonomy/term', 'node') === 0)) {
      // It is an alias, so convert it to a system path.
      $path = \Drupal::service('path.alias_manager')->getPathByAlias($path);
    }
    $params = Url::fromUri("internal:{$path}")->getRouteParameters();
    $entity_type = key($params);
    $entity = \Drupal::entityTypeManager()->getStorage($entity_type)->load($params[$entity_type]);
    if (($entity instanceof ContentEntityInterface) && (!$entity->isPublished())) {
      // It is an unpublished entity.
      return TRUE;
    }
    else {
      // It is something other than an unpublished.
      return FALSE;
    }
  }


  /**
   * Get the html from the node contents.
   *
   * @param NodeInterface
   *  The node object to generate the html from.
   *
   * @return string
   *   The html of the node content.
   */
  private function getNodeHtml($node) {
    $builder = \Drupal::entityTypeManager()->getViewBuilder('node');
    // Use a bogus view mode so that no view mode will be used.
    // This might be a mistake, but seems to be working.
    $view_mode = 'bogus-view-mode-12345-giberish';
    $build = $builder->view($node, $view_mode);
    $html = render($build);

    return $html;
  }

  /**
   * Checks if this is node preview.
   *
   * @return bool
   *   TRUE if it is node preview.  FALSE otherwise.
   */
  private function isPreview() {
    $route_name = \Drupal::routeMatch()->getRouteName();
    return ($route_name === "entity.node.preview") ? TRUE : FALSE;
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    // CHeck to see if we are on node/NID. node/NID/editor node preview.
    $route_name = \Drupal::routeMatch()->getRouteName();
    $allowed_routes = [];
    if ($this->nlr_config->get('display_on_node_view')) {
      $allowed_routes[] = 'entity.node.canonical';
    }
    if ($this->nlr_config->get('display_on_node_edit')) {
      $allowed_routes[] = 'entity.node.edit_form';
    }
    if ($this->nlr_config->get('display_on_node_preview')) {
      $allowed_routes[] = 'entity.node.preview';
    }

    if (in_array($route_name, $allowed_routes)) {
      return AccessResult::allowedIfHasPermission($account, 'view node link report');
    }
    else {
      // Not on the right path, hide the block.
      return AccessResult::forbiddenIf(TRUE);
    }

  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return 0;
  }

  /**
   * Checks if the URL can be tested.
   *
   * @param string $url
   *   An url to be evaluated.  Relative urls will be converted to absolute.
   *
   * @return bool
   *   TRUE if the url should be tested.  FALSE otherwise.
   */
  private function isAllowed(&$url) {
    $process_external = $this->nlr_config->get('enable_check_external_links');
    // Check for mailto.
    $scheme = strtolower(parse_url($url, PHP_URL_SCHEME));
    $non_curlable = ['mailto', 'im', 'sms', 'tel'];
    if (in_array($scheme, $non_curlable)) {
      // This is not curl-able but is a valid scheme.
      return FALSE;
    }
    $this->convertRelativeToFull($url);
    if ($this->isExternal($url)) {
      // Is external, see if we should process.
      return ($process_external) ? TRUE : FALSE;
    }
    else {
      // Is internal.
      return TRUE;
    }
  }

  /**
   * Convert relative links to absolute links.
   *
   * @param string $url
   *   Url by reference to make a full url if it is a relative URL.
   */
  private function convertRelativeToFull(&$url) {
    // Make the url slashless for consistency.
    $url = ltrim($url, '/');
    $schema = parse_url($url, PHP_URL_SCHEME);
    if (empty($schema)) {
      // Assume this is relative url. (Some risk here.)
      $currentSchemaAndHost = \Drupal::request()->getSchemeAndHttpHost();
      $url = "{$currentSchemaAndHost}/{$url}";
    }
  }

  /**
   * Evaluates if a url is internal.
   *
   * @param string $url
   *   An URL to check.
   *
   * @return bool
   *   TRUE if the URL is external to the site.
   */
  private function isExternal($url) {
    $domain_list = $this->nlr_config->get('additional_domains_as_internal');
    // Clean up any bad data by converting all separators into ','.
    $domain_list = str_replace(["\r\n", "\n", "\r", ', ', ' ,', " "], ',', $domain_list);
    // Clean up by removing any scheme.
    $domain_list = str_replace(['http://', 'https://'], '', $domain_list);
    $internal_domains = explode(',', $domain_list);
    // Add the actual host.
    $internal_domains[] = \Drupal::request()->getHost();
    $url_host = parse_url($url, PHP_URL_HOST);
    return (in_array($url_host, $internal_domains)) ? FALSE : TRUE;
  }

  /**
   * Getter and setter for the current node.
   *
   * @return NodeInterface
   *   Returns the node for the current route. or NULL if not found.
   */
  protected function getNode() {
    if (empty($this->node)) {
      $route_name = \Drupal::routeMatch()->getRouteName();

      if (($route_name === 'entity.node.canonical') || ($route_name === 'entity.node.edit_form')) {
        $node = \Drupal::routeMatch()->getParameter('node');
      } elseif ($route_name === 'entity.node.preview') {
        $node = \Drupal::routeMatch()->getParameter('node_preview');
      }

      if ($node instanceof NodeInterface) {
        $this->node = $node;
      }
    }
    return $this->node;
  }

  /**
   * Curls a url and returns TRUE if the url is found.
   *
   * @param string $url
   *   A url to curl.
   *
   * @return bool
   *   TRUE if the url returned a 200, FALSE otherwise.
   */
  private function curlUrl($url) {
    try {
      $ch = curl_init($url);
      // We just want the header.
      curl_setopt($ch, CURLOPT_HEADER, TRUE);
      // We don't need the body.
      curl_setopt($ch, CURLOPT_NOBODY, TRUE);
      // Catch output, do not print.
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
      // Follow redirects.
      curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
      // Allow up to 10 redirects to not get stuck for too long.
      curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
      // Prevent waiting forever to get a result
      curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
      curl_setopt($ch, CURLOPT_TIMEOUT, 6);
      curl_setopt($ch, CURLOPT_USERAGENT, "Mozilla/5.0 (Windows NT 5.1; rv:31.0) Gecko/20100101 Firefox/31.0");

      curl_exec($ch);
      $returncode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
      curl_close($ch);
      if ($returncode === 200) {
        return TRUE;
      } else {
        return FALSE;
      }
    } catch (\Exception $e) {
      watchdog_exception('node_link_report', $e);
      return FALSE;
    }
  }
}

==> node_link_report-8.x-1.2.tar.gz: node_link_report/src/Plugin/Block/NodeLinkReport.php <==
<?php

namespace Drupal\node_link_report\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\node\NodeInterface;

/**
 * Provides a Node Report Block.
 *
 * @Block(
 *   id = "node_link_report_block",
 *   admin_label = @Translation("Node Link Report"),
 * )
 */
class NodeLinkReport extends BlockBase {

  /**
   * @var object $nlr_config
   *  Configuration settings object for node_link_report module.
   */
private $nlr_config;

/**
 * @var NodeInterface $node
 *   The node object for the current node.
 */
private $node = NULL;

  /**
   * NodeLinkReport constructor.
   */
  public function __construct() {
    $this->nlr_config = \Drupal::config('node_link_report.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $nid = $this->getNode()->id();
    $cid = "node_link_report_{$nid}";
    // Check for cached data.
    $cached_data  = \Drupal::cache()->get($cid);
    if (!empty($cached_data) && (!$this->isPreview())) {
      $renderable = $cached_data->data;
    }
    else {
      // Cache unavailable so generate the data.
      $links = $this->checkLinks();
      $renderable = [
        '#theme' => 'node_link_report_block',
        '#node_id' => $nid,
        '#bad_links' => $links['bad'],
        '#display_good_links' => $this->nlr_config->get('enable_reporting_good_links'),
        '#good_links' => $links['good'],
        '#unpublished_links' => $links['unpublished'],
      ];

      if (!$this->isPreview()) {
        // This is not a preview, so cache the data. for 24 hours.
        $expire_time = time() + 24 * 60 * 60;
        $tags = [
          'node_link_report',
          "node:{$nid}",
        ];
        \Drupal::cache()->set($cid, $renderable, $expire_time, $tags);
      }
    }

    $rendered = \Drupal::service('renderer')->render($renderable);
    return [
      '#markup' => $rendered,
    ];
  }

  /**
   * Checks the links of any found in the node content.
   *
   * @return array
   *   Array with the following keys 'bad', 'good', and 'unpublished'.
   */
  private function checkLinks() {
    $links = [
      'bad' => [],
      'good' => [],
      'unpublished' => [],
    ];
    $node = clone $this->getNode();
    $html = $this->getNodeHtml($node);
    //Instantiate the DOMDocument class.
    $htmlDom = new \DOMDocument;
    //Parse the HTML of the page using DOMDocument::loadHTML
    @$htmlDom->loadHTML($html);

    //Extract the links from the HTML.
    $urls = $htmlDom->getElementsByTagName('a');
    //Dedupe the urls, by url.
    $deduped_urls = [];
    foreach ($urls as $url) {
      //Get the link text.
      $linkText = $url->nodeValue;
      //Get the link in the href attribute.
      $linkHref = $url->getAttribute('href');
      if (!empty($linkHref) && $this->isAllowed($linkHref)) {
        if (isset($deduped_urls[$linkHref])) {
          $deduped_urls[$linkHref] .= "[{$linkText}] ";
        }
        else {
          $deduped_urls[$linkHref] = "[{$linkText}] ";
        }

      }
    }
    foreach ($deduped_urls as $url => $text) {
      // Curl the href to get the response.
      $exists = $this->curlUrl($url);
      if ($exists) {
        // The url is accessible.
        $links['good'][$url] = $text;
      }
      else {
        // The url is not accessible and is either bad or unpublished.
        if ($this->isExternal($url)) {
          // This is definitely external and bad.
          $links['bad'][$url] = $text;
        }
        else {
          // The internal link may be bad or unpublished.
          if ($this->isUnpublished($url)) {
            // It is an url to an unpublished page.
            $links['unpublished'][$url] = $text;
          }
          else {
            // It is not unpublished, just bad.
            $links['bad'][$url] = $text;
          }
        }
      }
    }

    return $links;
  }

  /**
   * Looks up to see if an internal link is unpublished.
   *
   * @param string $url
   *   A url to lookup.
   *
   * @return bool
   *   TRUE if the link points to an unpublished entity.  FALSE otherwise.
   */
  private function isUnpublished($url) {
    // Look at the path.
    $path = parse_url($url, PHP_URL_PATH);
    // Determine if it is a node/NID path.
    if (!(strpos($path, 'node') === 0) || !(strpos($path, 'taxonomy/term') === 0)) {
      // It is an alias, so convert it to a system path.
      $path = \Drupal::service('path.alias_manager')->getPathByAlias($path);
    }
    $is_valid = \Drupal::service('path.validator')->isValid($path);
    if ($is_valid) {
      $params = Url::fromUri("internal:{$path}")->getRouteParameters();
      $entity_type = key($params);
      $entity = \Drupal::entityTypeManager()->getStorage($entity_type)->load($params[$entity_type]);
      if (($entity instanceof ContentEntityInterface) && (method_exists($entity, 'isPublished')) && (!$entity->isPublished())) {
        // It is an unpublished entity.
        return TRUE;
      } else {
        // It is something other than an unpublished.
        return FALSE;
      }
    }
    else {
      // Not a valid path.
      return FALSE;
    }

  }


  /**
   * Get the html from the node contents.
   *
   * @param NodeInterface
   *  The node object to generate the html from.
   *
   * @return string
   *   The html of the node content.
   */
  private function getNodeHtml($node) {
    $builder = \Drupal::entityTypeManager()->getViewBuilder('node');
    // Use a bogus view mode so that no view mode will be used.
    // This might be a mistake, but seems to be working.
    $view_mode = 'bogus-view-mode-12345-giberish';
    $build = $builder->view($node, $view_mode);
    $html = render($build);

    return $html;
  }

  /**
   * Checks if this is node preview.
   *
   * @return bool
   *   TRUE if it is node preview.  FALSE otherwise.
   */
  private function isPreview() {
    $route_name = \Drupal::routeMatch()->getRouteName();
    return ($route_name === "entity.node.preview") ? TRUE : FALSE;
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    // CHeck to see if we are on node/NID. node/NID/editor node preview.
    $route_name = \Drupal::routeMatch()->getRouteName();
    $allowed_routes = [];
    if ($this->nlr_config->get('display_on_node_view')) {
      $allowed_routes[] = 'entity.node.canonical';
    }
    if ($this->nlr_config->get('display_on_node_edit')) {
      $allowed_routes[] = 'entity.node.edit_form';
    }
    if ($this->nlr_config->get('display_on_node_preview')) {
      $allowed_routes[] = 'entity.node.preview';
    }

    if (in_array($route_name, $allowed_routes)) {
      return AccessResult::allowedIfHasPermission($account, 'view node link report');
    }
    else {
      // Not on the right path, hide the block.
      return AccessResult::forbiddenIf(TRUE);
    }

  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return 0;
  }

  /**
   * Checks if the URL can be tested.
   *
   * @param string $url
   *   An url to be evaluated.  Relative urls will be converted to absolute.
   *
   * @return bool
   *   TRUE if the url should be tested.  FALSE otherwise.
   */
  private function isAllowed(&$url) {
    $process_external = $this->nlr_config->get('enable_check_external_links');
    // Check for mailto.
    $scheme = strtolower(parse_url($url, PHP_URL_SCHEME));
    $non_curlable = ['mailto', 'im', 'sms', 'tel'];
    if (in_array($scheme, $non_curlable)) {
      // This is not curl-able but is a valid scheme.
      return FALSE;
    }
    $this->convertRelativeToFull($url);
    if ($this->isExternal($url)) {
      // Is external, see if we should process.
      return ($process_external) ? TRUE : FALSE;
    }
    else {
      // Is internal.
      return TRUE;
    }
  }

  /**
   * Convert relative links to absolute links.
   *
   * @param string $url
   *   Url by reference to make a full url if it is a relative URL.
   */
  private function convertRelativeToFull(&$url) {
    // Make the url slashless for consistency.
    $url = ltrim($url, '/');
    $schema = parse_url($url, PHP_URL_SCHEME);
    if (empty($schema)) {
      // Assume this is relative url. (Some risk here.)
      $currentSchemaAndHost = \Drupal::request()->getSchemeAndHttpHost();
      $url = "{$currentSchemaAndHost}/{$url}";
    }
  }

  /**
   * Evaluates if a url is internal.
   *
   * @param string $url
   *   An URL to check.
   *
   * @return bool
   *   TRUE if the URL is external to the site.
   */
  private function isExternal($url) {
    $domain_list = $this->nlr_config->get('additional_domains_as_internal');
    // Clean up any bad data by converting all separators into ','.
    $domain_list = str_replace(["\r\n", "\n", "\r", ', ', ' ,', " "], ',', $domain_list);
    // Clean up by removing any scheme.
    $domain_list = str_replace(['http://', 'https://'], '', $domain_list);
    $internal_domains = explode(',', $domain_list);
    // Add the actual host.
    $internal_domains[] = \Drupal::request()->getHost();
    $url_host = parse_url($url, PHP_URL_HOST);
    return (in_array($url_host, $internal_domains)) ? FALSE : TRUE;
  }

  /**
   * Getter and setter for the current node.
   *
   * @return NodeInterface
   *   Returns the node for the current route. or NULL if not found.
   */
  protected function getNode() {
    if (empty($this->node)) {
      $route_name = \Drupal::routeMatch()->getRouteName();

      if (($route_name === 'entity.node.canonical') || ($route_name === 'entity.node.edit_form')) {
        $node = \Drupal::routeMatch()->getParameter('node');
      } elseif ($route_name === 'entity.node.preview') {
        $node = \Drupal::routeMatch()->getParameter('node_preview');
      }

      if ($node instanceof NodeInterface) {
        $this->node = $node;
      }
    }
    return $this->node;
  }

  /**
   * Curls a url and returns TRUE if the url is found.
   *
   * @param string $url
   *   A url to curl.
   *
   * @return bool
   *   TRUE if the url returned a 200, FALSE otherwise.
   */
  private function curlUrl($url) {
    try {
      $ch = curl_init($url);
      // We just want the header.
      curl_setopt($ch, CURLOPT_HEADER, TRUE);
      // We don't need the body.
      curl_setopt($ch, CURLOPT_NOBODY, TRUE);
      // Catch output, do not print.
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
      // Follow redirects.
      curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
      // Allow up to 10 redirects to not get stuck for too long.
      curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
      // Prevent waiting forever to get a result
      curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
      curl_setopt($ch, CURLOPT_TIMEOUT, 10);
      curl_setopt($ch, CURLOPT_USERAGENT, "Mozilla/5.0 (Windows NT 5.1; rv:31.0) Gecko/20100101 Firefox/31.0");
      curl_exec($ch);
      $returncode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
      curl_close($ch);
      $acceptable_return_codes = [200, 301, 302, 307, 308];
      if (in_array($returncode, $acceptable_return_codes)) {
        return TRUE;
      } else {
        return FALSE;
      }
    } catch (\Exception $e) {
      watchdog_exception('node_link_report', $e);
      return FALSE;
    }
  }
}
